import "Turbine"
import "Turbine.UI"
import "Turbine.UI.Lotro"
import "YourName.PartySample.PartyWrapper"

function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            local i;
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

sampleWindow=Turbine.UI.Lotro.Window();
sampleWindow:SetBackColor(Turbine.UI.Color(0,0,0,0));
sampleWindow:SetSize(400,400);
sampleWindow:SetText("Party Sample")
sampleWindow.PartyList=Turbine.UI.ListBox();
sampleWindow.PartyList:SetParent(sampleWindow);
sampleWindow.PartyList:SetSize(sampleWindow:GetWidth()-32,sampleWindow:GetHeight()-110);
sampleWindow.PartyList:SetPosition(10,50);
sampleWindow.PartyList:SetBackColor(Turbine.UI.Color(.1,.1,.1));

sampleWindow.VScroll=Turbine.UI.Lotro.ScrollBar();
sampleWindow.VScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
sampleWindow.VScroll:SetParent(sampleWindow);
sampleWindow.VScroll:SetPosition(sampleWindow:GetWidth()-22,50);
sampleWindow.VScroll:SetWidth(12);
sampleWindow.VScroll:SetHeight(sampleWindow.PartyList:GetHeight());
sampleWindow.PartyList:SetVerticalScrollBar(sampleWindow.VScroll);

sampleWindow.Count=Turbine.UI.Label();
sampleWindow.Count:SetParent(sampleWindow);
sampleWindow.Count:SetSize(200,20);
sampleWindow.Count:SetPosition(sampleWindow:GetWidth()/2-100,sampleWindow:GetHeight()-55);
sampleWindow.Count:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
sampleWindow.Count:SetText("Count:0");

sampleWindow.RefreshButton=Turbine.UI.Lotro.Button();
sampleWindow.RefreshButton:SetParent(sampleWindow);
sampleWindow.RefreshButton:SetSize(150,20);
sampleWindow.RefreshButton:SetPosition(sampleWindow:GetWidth()/2-75,sampleWindow:GetHeight()-30);
sampleWindow.RefreshButton:SetText("Force Refresh");
sampleWindow.RefreshButton.MouseClick=function()
	-- force a refresh of the wrapper
	PartyWrapper.loaded=false;
	PartyWrapper.initialized=false;
	PartyWrapper:SetWantsUpdates(true);
	sampleWindow:SetWantsUpdates(true);
end

sampleWindow.RefreshList=function()
	sampleWindow.PartyList:ClearItems();
	local count=PartyWrapper:GetMemberCount();
	sampleWindow.Count:SetText("Count:"..tonumber(count));
	if count>0 then
		local leader=PartyWrapper:GetLeaderName();
		local tmpIndex;
		for tmpIndex=1,count do
			local tmpRow=Turbine.UI.Label();
			tmpRow:SetParent(sampleWindow.PartyList);
			tmpRow:SetSize(sampleWindow.PartyList:GetWidth(),20);
			local name=PartyWrapper:GetMemberName(tmpIndex);
			if name==leader then
				tmpRow:SetForeColor(Turbine.UI.Color(0,1,0));
			else
				tmpRow:SetForeColor(Turbine.UI.Color(0,.2,1));
			end
			tmpRow:SetText(name);
			sampleWindow.PartyList:AddItem(tmpRow);
		end
	end
end
sampleWindow.loaded=false;
sampleWindow.Update=function()
	if Plugins["PartySample"]~=nil and sampleWindow.loaded==false then
		sampleWindow.loaded=true;
		Plugins["PartySample"].Unload=function()
			RemoveCallback(PartyWrapper, "LeaderChanged", LeaderChanged);
			RemoveCallback(PartyWrapper, "MemberAdded", MemberAdded);
			RemoveCallback(PartyWrapper, "MemberRemoved", MemberRemoved);
			RemoveCallback(PartyWrapper, "JoinedParty", JoinedParty);
			RemoveCallback(PartyWrapper, "LeftParty", LeftParty);
			PartyWrapper:Unload(); -- this will unregister the chat event handler - this assumes we are the only plugin using the wrapper... should change this to allow for other plugins
			Turbine.Shell.RemoveCommand(sampleWindow.shellCommand);
		end
	end
	if PartyWrapper.initialized then
		sampleWindow.loaded=true;
		sampleWindow:SetWantsUpdates(false);
		sampleWindow:RefreshList();
	end
end

LeaderChanged=function(sender,args)
	sampleWindow:RefreshList();
end

MemberAdded=function()
	sampleWindow:RefreshList();
end
MemberRemoved=function()
	sampleWindow:RefreshList();
end
JoinedParty=function()
	PartyWrapper.loaded=false;
	PartyWrapper.initialized=false;
	PartyWrapper:SetWantsUpdates(true);
	sampleWindow:SetWantsUpdates(true);
end
LeftParty=function()
	sampleWindow:RefreshList();
end
AddCallback(PartyWrapper, "LeaderChanged", LeaderChanged);
AddCallback(PartyWrapper, "MemberAdded", MemberAdded);
AddCallback(PartyWrapper, "MemberRemoved", MemberRemoved);
AddCallback(PartyWrapper, "JoinedParty", JoinedParty);
AddCallback(PartyWrapper, "LeftParty", LeftParty);

sampleWindow.shellCommand=Turbine.ShellCommand();
sampleWindow.shellCommand.Execute = function(sender, cmd, args)
	if string.lower(args)=="toggle" then
		sampleWindow:SetVisible(not sampleWindow:IsVisible());
	end
end
Turbine.Shell.AddCommand("partySample",sampleWindow.shellCommand);


sampleWindow:SetWantsUpdates(true);
sampleWindow:SetVisible(true);